@@ -38,7 +38,7 @@
 #include <pjsip_ua.h>
 #include <pjlib.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 428302 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 431468 $")
 
 #include "asterisk/lock.h"
 #include "asterisk/channel.h"
@@ -490,7 +490,6 @@ static int answer(void *data)
 	struct ast_sip_session *session = data;
 
 	if (session->inv_session->state == PJSIP_INV_STATE_DISCONNECTED) {
-		ao2_ref(session, -1);
 		return 0;
 	}
 
@@ -507,8 +506,6 @@ static int answer(void *data)
 		ast_sip_session_send_response(session, packet);
 	}
 
-	ao2_ref(session, -1);
-
 	return (status == PJ_SUCCESS) ? 0 : -1;
 }
 
@@ -516,19 +513,27 @@ static int answer(void *data)
 static int chan_pjsip_answer(struct ast_channel *ast)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(ast);
+	struct ast_sip_session *session;
 
 	if (ast_channel_state(ast) == AST_STATE_UP) {
 		return 0;
 	}
 
 	ast_setstate(ast, AST_STATE_UP);
+	session = ao2_bump(channel->session);
 
-	ao2_ref(channel->session, +1);
-	if (ast_sip_push_task(channel->session->serializer, answer, channel->session)) {
+	/* the answer task needs to be pushed synchronously otherwise a race condition
+	   can occur between this thread and bridging (specifically when native bridging
+	   attempts to do direct media) */
+	ast_channel_unlock(ast);
+	if (ast_sip_push_task_synchronous(session->serializer, answer, session)) {
 		ast_log(LOG_WARNING, "Unable to push answer task to the threadpool. Cannot answer call\n");
-		ao2_cleanup(channel->session);
+		ao2_ref(session, -1);
+		ast_channel_lock(ast);
 		return -1;
 	}
+	ao2_ref(session, -1);
+	ast_channel_lock(ast);
 
 	return 0;
 }
@@ -1687,22 +1692,7 @@ static int hangup(void *data)
 	struct ast_sip_session *session = channel->session;
 	int cause = h_data->cause;
 
-	if (!session->defer_terminate) {
-		pj_status_t status;
-		pjsip_tx_data *packet = NULL;
-
-		if (session->inv_session->state == PJSIP_INV_STATE_NULL) {
-			pjsip_inv_terminate(session->inv_session, cause ? cause : 603, PJ_TRUE);
-		} else if (((status = pjsip_inv_end_session(session->inv_session, cause ? cause : 603, NULL, &packet)) == PJ_SUCCESS)
-			&& packet) {
-			if (packet->msg->type == PJSIP_RESPONSE_MSG) {
-				ast_sip_session_send_response(session, packet);
-			} else {
-				ast_sip_session_send_request(session, packet);
-			}
-		}
-	}
-
+	ast_sip_session_terminate(session, cause);
 	clear_session_and_channel(session, ast, pvt);
 	ao2_cleanup(channel);
 	ao2_cleanup(h_data);
@@ -2024,6 +2014,21 @@ static int chan_pjsip_incoming_request(struct ast_sip_session *session, struct p
 		return 0;
 	}
 
+	if (session->inv_session->state >= PJSIP_INV_STATE_CONFIRMED) {
+		/* Weird case. We've received a reinvite but we don't have a channel. The most
+		 * typical case for this happening is that a blind transfer fails, and so the
+		 * transferer attempts to reinvite himself back into the call. We already got
+		 * rid of that channel, and the other side of the call is unrecoverable.
+		 *
+		 * We treat this as a failure, so our best bet is to just hang this call
+		 * up and not create a new channel. Clearing defer_terminate here ensures that
+		 * calling ast_sip_session_terminate() can result in a BYE being sent ASAP.
+		 */
+		session->defer_terminate = 0;
+		ast_sip_session_terminate(session, 400);
+		return -1;
+	}
+
 	datastore = ast_sip_session_alloc_datastore(&transport_info, "transport_info");
 	if (!datastore) {
 		return -1;
@@ -2053,7 +2058,7 @@ static int chan_pjsip_incoming_request(struct ast_sip_session *session, struct p
 
 static int call_pickup_incoming_request(struct ast_sip_session *session, pjsip_rx_data *rdata)
 {
-	struct ast_features_pickup_config *pickup_cfg = ast_get_chan_features_pickup_config(session->channel);
+	struct ast_features_pickup_config *pickup_cfg;
 	struct ast_channel *chan;
 
 	/* We don't care about reinvites */
@@ -2061,6 +2066,7 @@ static int call_pickup_incoming_request(struct ast_sip_session *session, pjsip_r
 		return 0;
 	}
 
+	pickup_cfg = ast_get_chan_features_pickup_config(session->channel);
 	if (!pickup_cfg) {
 		ast_log(LOG_ERROR, "Unable to retrieve pickup configuration options. Unable to detect call pickup extension.\n");
 		return 0;
