@@ -984,7 +984,17 @@ ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (media_encryption_opt
 
 /
 
-INSERT INTO alembic_version (version_num) VALUES ('eb88a14f2a')
+-- Running upgrade eb88a14f2a -> 371a3bf4143e
+
+ALTER TABLE ps_endpoints ADD user_eq_phone VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (user_eq_phone IN ('yes', 'no'))
+
+/
+
+INSERT INTO alembic_version (version_num) VALUES ('371a3bf4143e')
 
 /
 
