@@ -91,6 +91,8 @@ struct ast_sip_transport {
 	AST_DECLARE_STRING_FIELDS(
 		/*! Certificate of authority list file */
 		AST_STRING_FIELD(ca_list_file);
+		/*! Certificate of authority list path */
+		AST_STRING_FIELD(ca_list_path);
 		/*! Public certificate file */
 		AST_STRING_FIELD(cert_file);
 		/*! Optional private key of the certificate file */
@@ -609,6 +611,8 @@ struct ast_sip_endpoint {
 	enum ast_sip_session_redirect redirect_method;
 	/*! Variables set on channel creation */
 	struct ast_variable *channel_vars;
+	/*! Whether to place a 'user=phone' parameter into the request URI if user is a number */
+	unsigned int usereqphone;
 };
 
 /*!
@@ -913,6 +917,16 @@ struct ao2_container *ast_sip_location_retrieve_aor_contacts(const struct ast_si
  */
 struct ast_sip_contact *ast_sip_location_retrieve_contact_from_aor_list(const char *aor_list);
 
+/*!
+ * \brief Retrieve the first bound contact AND the AOR chosen from a list of AORs
+ *
+ * \param aor_list A comma-separated list of AOR names
+ * \param aor The chosen AOR
+ * \param contact The chosen contact
+ */
+ void ast_sip_location_retrieve_contact_and_aor_from_list(const char *aor_list, struct ast_sip_aor **aor,
+	struct ast_sip_contact **contact);
+
 /*!
  * \brief Retrieve a named contact
  *
@@ -1486,6 +1500,15 @@ void ast_copy_pj_str(char *dest, const pj_str_t *src, size_t size);
  */
 struct ast_sip_endpoint *ast_pjsip_rdata_get_endpoint(pjsip_rx_data *rdata);
 
+/*!
+ * \brief Add 'user=phone' parameter to URI if enabled and user is a phone number.
+ *
+ * \param endpoint The endpoint to use for configuration
+ * \param pool The memory pool to allocate the parameter from
+ * \param uri The URI to check for user and to add parameter to
+ */
+void ast_sip_add_usereqphone(const struct ast_sip_endpoint *endpoint, pj_pool_t *pool, pjsip_uri *uri);
+
 /*!
  * \brief Retrieve any endpoints available to sorcery.
  *
@@ -1956,4 +1979,11 @@ char *ast_sip_get_debug(void);
 		}						\
 	} while(0)
 
+/*!
+ * \brief Retrieve the system keep alive interval setting.
+ *
+ * \retval the keep alive interval.
+ */
+unsigned int ast_sip_get_keep_alive_interval(void);
+
 #endif /* _RES_PJSIP_H */
