@@ -105,6 +105,8 @@ struct ast_sip_session {
 	char exten[AST_MAX_EXTENSION];
 	/*! The endpoint with which Asterisk is communicating */
 	struct ast_sip_endpoint *endpoint;
+	/*! The AOR associated with this session */
+	struct ast_sip_aor *aor;
 	/*! The contact associated with this session */
 	struct ast_sip_contact *contact;
 	/*! The PJSIP details of the session, which includes the dialog */
@@ -433,6 +435,14 @@ struct ast_sip_session *ast_sip_session_create_outgoing(struct ast_sip_endpoint
 	struct ast_sip_contact *contact, const char *location, const char *request_user,
 	struct ast_format_cap *req_caps);
 
+/*!
+ * \brief Terminate a session and, if possible, send the provided response code
+ *
+ * \param session The session to terminate
+ * \param response The response code to use for termination if possible
+ */
+void ast_sip_session_terminate(struct ast_sip_session *session, int response);
+
 /*!
  * \brief Defer local termination of a session until remote side terminates, or an amount of time passes
  *
