@@ -30,7 +30,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 429064 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 431468 $")
 
 #include "asterisk/astobj2.h"
 #include "asterisk/json.h"
@@ -888,7 +888,7 @@ struct ast_json *ast_channel_snapshot_to_json(
 		/* Broken up into groups of three for readability */
 		"{ s: s, s: s, s: s,"
 		"  s: o, s: o, s: s,"
-		"  s: o, s: o }",
+		"  s: o, s: o, s: s }",
 		/* First line */
 		"id", snapshot->uniqueid,
 		"name", snapshot->name,
@@ -902,7 +902,8 @@ struct ast_json *ast_channel_snapshot_to_json(
 		/* Third line */
 		"dialplan", ast_json_dialplan_cep(
 			snapshot->context, snapshot->exten, snapshot->priority),
-		"creationtime", ast_json_timeval(snapshot->creationtime, NULL));
+		"creationtime", ast_json_timeval(snapshot->creationtime, NULL),
+		"language", snapshot->language);
 
 	return ast_json_ref(json_chan);
 }
@@ -1003,13 +1004,14 @@ static struct ast_json *dtmf_end_to_json(
 	const char *direction =
 		ast_json_string_get(ast_json_object_get(blob, "direction"));
 	const struct timeval *tv = stasis_message_timestamp(message);
-	struct ast_json *json_channel = ast_channel_snapshot_to_json(snapshot, sanitize);
+	struct ast_json *json_channel;
 
 	/* Only present received DTMF end events as JSON */
 	if (strcasecmp("Received", direction) != 0) {
 		return NULL;
 	}
 
+	json_channel = ast_channel_snapshot_to_json(snapshot, sanitize);
 	if (!json_channel) {
 		return NULL;
 	}
