@@ -424,7 +424,7 @@ static void send_unsolicited_mwi_notify(struct mwi_subscription *sub,
 
 		contacts = ast_sip_location_retrieve_aor_contacts(aor);
 		if (!contacts || (ao2_container_count(contacts) == 0)) {
-			ast_log(LOG_WARNING, "No contacts bound to AOR %s. Cannot send unsolicited MWI.\n", aor_name);
+			ast_log(LOG_NOTICE, "No contacts bound to AOR %s. Cannot send unsolicited MWI until a contact registers.\n", aor_name);
 			continue;
 		}
 
@@ -658,7 +658,7 @@ static struct mwi_subscription *mwi_subscribe_all(
 static int mwi_new_subscribe(struct ast_sip_endpoint *endpoint,
 		const char *resource)
 {
-	struct ast_sip_aor *aor;
+	RAII_VAR(struct ast_sip_aor *, aor, NULL, ao2_cleanup);
 
 	if (ast_strlen_zero(resource)) {
 		if (ast_sip_for_each_aor(endpoint->aors, mwi_validate_for_aor, endpoint)) {
@@ -676,7 +676,7 @@ static int mwi_new_subscribe(struct ast_sip_endpoint *endpoint,
 	}
 
 	if (ast_strlen_zero(aor->mailboxes)) {
-		ast_log(LOG_WARNING, "AOR %s has no configured mailboxes. "
+		ast_log(LOG_NOTICE, "AOR %s has no configured mailboxes. "
 			"MWI subscription failed\n", resource);
 		return 404;
 	}
