@@ -242,16 +242,16 @@ static struct ast_frame *refer_progress_framehook(struct ast_channel *chan, stru
 
 	/* If a notification is due to be sent push it to the thread pool */
 	if (notification) {
-		if (ast_sip_push_task(progress->serializer, refer_progress_notify, notification)) {
-			ao2_cleanup(notification);
-		}
-
 		/* If the subscription is being terminated we don't need the frame hook any longer */
 		if (notification->state == PJSIP_EVSUB_STATE_TERMINATED) {
 			ast_debug(3, "Detaching REFER progress monitoring hook from '%s' as subscription is being terminated\n",
 				ast_channel_name(chan));
 			ast_framehook_detach(chan, progress->framehook);
 		}
+
+		if (ast_sip_push_task(progress->serializer, refer_progress_notify, notification)) {
+			ao2_cleanup(notification);
+		}
 	}
 
 	return f;
@@ -420,6 +420,7 @@ static void refer_attended_destroy(void *obj)
 	ao2_cleanup(attended->transferer);
 	ast_channel_unref(attended->transferer_chan);
 	ao2_cleanup(attended->transferer_second);
+	ao2_cleanup(attended->progress);
 }
 
 /*! \brief Allocator for attended transfer task */
@@ -599,9 +600,16 @@ static void refer_blind_callback(struct ast_channel *chan, struct transfer_chann
 
 	if (refer->replaces) {
 		char replaces[512];
+		char *replaces_val = NULL;
+		int len;
 
-		pjsip_hdr_print_on(refer->replaces, replaces, sizeof(replaces));
-		pbx_builtin_setvar_helper(chan, "__SIPREPLACESHDR", S_OR(replaces, NULL));
+		len = pjsip_hdr_print_on(refer->replaces, replaces, sizeof(replaces) - 1);
+		if (len != -1) {
+			/* pjsip_hdr_print_on does not NULL terminate the buffer */
+			replaces[len] = '\0';
+			replaces_val = replaces + sizeof("Replaces:");
+		}
+		pbx_builtin_setvar_helper(chan, "__SIPREPLACESHDR", replaces_val);
 	} else {
 		pbx_builtin_setvar_helper(chan, "SIPREPLACESHDR", NULL);
 	}
@@ -674,8 +682,8 @@ static int refer_incoming_attended_request(struct ast_sip_session *session, pjsi
 		}
 
 		if (!ast_exists_extension(NULL, context, "external_replaces", 1, NULL)) {
-			ast_log(LOG_ERROR, "Received REFER for remote session on channel '%s' from endpoint '%s' but 'external_replaces' context does not exist for handling\n",
-				ast_channel_name(session->channel), ast_sorcery_object_get_id(session->endpoint));
+			ast_log(LOG_ERROR, "Received REFER for remote session on channel '%s' from endpoint '%s' but 'external_replaces' extension not found in context %s\n",
+				ast_channel_name(session->channel), ast_sorcery_object_get_id(session->endpoint), context);
 			return 404;
 		}
 
