@@ -38,7 +38,7 @@
 #include <pjmedia.h>
 #include <pjlib.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 428222 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 431303 $")
 
 #include "asterisk/module.h"
 #include "asterisk/format.h"
@@ -119,16 +119,13 @@ static int create_rtp(struct ast_sip_session *session, struct ast_sip_session_me
 	ast_rtp_instance_set_prop(session_media->rtp, AST_RTP_PROPERTY_RTCP, 1);
 	ast_rtp_instance_set_prop(session_media->rtp, AST_RTP_PROPERTY_NAT, session->endpoint->media.rtp.symmetric);
 
-	if (session->endpoint->dtmf == AST_SIP_DTMF_INBAND) {
-		ast_rtp_instance_dtmf_mode_set(session_media->rtp, AST_RTP_DTMF_MODE_INBAND);
-	}
-
 	if (!session->endpoint->media.rtp.ice_support && (ice = ast_rtp_instance_get_ice(session_media->rtp))) {
 		ice->stop(session_media->rtp);
 	}
 
 	if (session->endpoint->dtmf == AST_SIP_DTMF_RFC_4733) {
 		ast_rtp_instance_dtmf_mode_set(session_media->rtp, AST_RTP_DTMF_MODE_RFC2833);
+		ast_rtp_instance_set_prop(session_media->rtp, AST_RTP_PROPERTY_DTMF, 1);
 	} else if (session->endpoint->dtmf == AST_SIP_DTMF_INBAND) {
 		ast_rtp_instance_dtmf_mode_set(session_media->rtp, AST_RTP_DTMF_MODE_INBAND);
 	}
@@ -1182,12 +1179,17 @@ static int apply_negotiated_sdp_stream(struct ast_sip_session *session, struct a
 
 	/* audio stream handles music on hold */
 	if (media_type != AST_MEDIA_TYPE_AUDIO) {
+		if ((pjmedia_sdp_neg_was_answer_remote(session->inv_session->neg) == PJ_FALSE)
+			&& (session->inv_session->state == PJSIP_INV_STATE_CONFIRMED)) {
+			ast_queue_control(session->channel, AST_CONTROL_UPDATE_RTP_PEER);
+		}
 		return 1;
 	}
 
 	if (ast_sockaddr_isnull(addrs) ||
 		ast_sockaddr_is_any(addrs) ||
-		pjmedia_sdp_media_find_attr2(remote_stream, "sendonly", NULL)) {
+		pjmedia_sdp_media_find_attr2(remote_stream, "sendonly", NULL) ||
+		pjmedia_sdp_media_find_attr2(remote_stream, "inactive", NULL)) {
 		if (!session_media->held) {
 			/* The remote side has put us on hold */
 			ast_queue_hold(session->channel, session->endpoint->mohsuggest);
@@ -1200,6 +1202,9 @@ static int apply_negotiated_sdp_stream(struct ast_sip_session *session, struct a
 		ast_queue_unhold(session->channel);
 		ast_queue_frame(session->channel, &ast_null_frame);
 		session_media->held = 0;
+	} else if ((pjmedia_sdp_neg_was_answer_remote(session->inv_session->neg) == PJ_FALSE)
+		&& (session->inv_session->state == PJSIP_INV_STATE_CONFIRMED)) {
+		ast_queue_control(session->channel, AST_CONTROL_UPDATE_RTP_PEER);
 	}
 
 	/* This purposely resets the encryption to the configured in case it gets added later */
@@ -1237,6 +1242,7 @@ static void stream_destroy(struct ast_sip_session_media *session_media)
 		ast_rtp_instance_stop(session_media->rtp);
 		ast_rtp_instance_destroy(session_media->rtp);
 	}
+	session_media->rtp = NULL;
 }
 
 /*! \brief SDP handler for 'audio' media stream */
