@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 429062 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 431450 $")
 
 #include "asterisk/bridge.h"
 #include "asterisk/bridge_after.h"
@@ -97,6 +97,56 @@ static void bridge_stasis_queue_join_action(struct ast_bridge *self,
 	ast_channel_unlock(bridge_channel->chan);
 }
 
+/*!
+ * \internal
+ * \brief Peek at channel before it is pushed into bridge
+ * \since 13.2.0
+ *
+ * \param self Bridge to operate upon.
+ * \param bridge_channel Bridge channel to push.
+ * \param swap Bridge channel to swap places with if not NULL.
+ *
+ * \note On entry, self is already locked.
+ *
+ * \retval 0 on success.
+ * \retval -1 on failure.  The channel should not be pushed.
+ */
+static int bridge_stasis_push_peek(struct ast_bridge *self, struct ast_bridge_channel *bridge_channel, struct ast_bridge_channel *swap)
+{
+	struct stasis_app_control *swap_control;
+	struct ast_channel_snapshot *to_be_replaced;
+
+	if (!swap) {
+		goto done;
+	}
+
+	swap_control = stasis_app_control_find_by_channel(swap->chan);
+	if (!swap_control) {
+		ast_log(LOG_ERROR,"Failed to find stasis app control for swapped channel %s\n", ast_channel_name(swap->chan));
+		return -1;
+	}
+	to_be_replaced = ast_channel_snapshot_get_latest(ast_channel_uniqueid(swap->chan));
+
+	ast_debug(3, "Copying stasis app name %s from %s to %s\n", app_name(control_app(swap_control)),
+		ast_channel_name(swap->chan), ast_channel_name(bridge_channel->chan));
+
+	ast_channel_lock(bridge_channel->chan);
+
+	/* copy the app name from the swap channel */
+	app_set_replace_channel_app(bridge_channel->chan, app_name(control_app(swap_control)));
+
+	/* set the replace channel snapshot */
+	app_set_replace_channel_snapshot(bridge_channel->chan, to_be_replaced);
+
+	ast_channel_unlock(bridge_channel->chan);
+
+	ao2_ref(swap_control, -1);
+	ao2_cleanup(to_be_replaced);
+
+done:
+	return ast_bridge_base_v_table.push_peek(self, bridge_channel, swap);
+}
+
 /*!
  * \internal
  * \brief Push this channel into the Stasis bridge.
@@ -125,8 +175,15 @@ static int bridge_stasis_push(struct ast_bridge *self, struct ast_bridge_channel
 		}
 
 		bridge_stasis_queue_join_action(self, bridge_channel);
+		if (swap) {
+			/* nudge the swap channel out of the bridge */
+			ast_bridge_channel_leave_bridge(swap, BRIDGE_CHANNEL_STATE_END_NO_DISSOLVE, 0);
+		}
 
-		/* Return -1 so the push fails and the after-bridge callback gets called */
+		/* Return -1 so the push fails and the after-bridge callback gets called
+		 * This keeps the bridging framework from putting the channel into the bridge
+		 * until the Stasis thread gets started, and then the channel is put into the bridge.
+		 */
 		return -1;
 	}
 
@@ -228,4 +285,5 @@ void bridge_stasis_init(void)
 	bridge_stasis_v_table.name = "stasis";
 	bridge_stasis_v_table.push = bridge_stasis_push;
 	bridge_stasis_v_table.pull = bridge_stasis_pull;
+	bridge_stasis_v_table.push_peek = bridge_stasis_push_peek;
 }
